﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using esridemo.Tools;
using ESRI.ArcGIS.Client;
using Microsoft.Phone.Shell;
using System.Windows.Controls.Primitives;
using System.ComponentModel;
using System.Threading;

namespace esridemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        private Identify _OpIdentify;
        private SpatialQuery _OpSpatialQuery;
        private TimeQuery _OpTimeQuery;
        private Analysis _OpAnalysis;
        private FeatureLayer _FLayer;
        private enum _MapOp
        {
            identify,
            spatialquery,
            timequery,
            analysis,
            no
        }
        private _MapOp _CurrentOp = _MapOp.no;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            ShowPopup();

            _FLayer = map1.Layers["BusinessLayer"] as FeatureLayer;
            _FLayer.UpdateCompleted += (s, a) =>
            {
                //Add MainPage to app level, so other pages can navigates from it.
                App app = Application.Current as App;
                if (!app.AppParameters.ContainsKey("MainPage"))
                    app.AppParameters.Add("MainPage", this);             
                _OpIdentify = new Identify(map1, _FLayer);
                _OpSpatialQuery = new SpatialQuery(map1, BusyIndicator);
                _OpTimeQuery = new TimeQuery(map1);
                _OpAnalysis = new Analysis(map1,this,BusyIndicator);
                _FLayer.Renderer = null;
            };
        }

        private void barbtn_Click(object sender, EventArgs e)
        {
            ApplicationBarIconButton barbtn=sender as ApplicationBarIconButton;
            switch (barbtn.Text)
            {
                case "店面信息":
                    ChangeMapOp(_MapOp.identify);
                    break;
                case "销售总额":
                    ChangeMapOp(_MapOp.spatialquery);
                    break;
                case "时间查询":
                    ChangeMapOp(_MapOp.timequery);
                    break;
                case "营业分析":
                    ChangeMapOp(_MapOp.analysis);
                    break;
                default:
                    break;
            }
        }

        private void barmenu_Click(object sender, EventArgs e)
        {
            ApplicationBarMenuItem mi = sender as ApplicationBarMenuItem;
            switch (mi.Text)
            {
                case "Bing Maps":                    
                    ChangeBaseMap(mi.Text);
                    mi.Text = "ArcGIS EMapZone";
                    break;
                case "ArcGIS EMapZone":                    
                    ChangeBaseMap(mi.Text);
                    mi.Text = "Bing Maps";
                    break;
                case "清空操作":
                    ChangeMapOp(_MapOp.no);
                    break;
                default:
                    break;
            }
        }

        private void ChangeMapOp(_MapOp op)
        {
            if (_CurrentOp == op)
                return;

            //disable last op first
            if (_CurrentOp == _MapOp.identify)
                _OpIdentify.IsActivated = false;
            else if (_CurrentOp == _MapOp.spatialquery)
                _OpSpatialQuery.IsActivated = false;
            else if (_CurrentOp == _MapOp.timequery)
                _OpTimeQuery.IsActivated = false;
            else if (_CurrentOp == _MapOp.analysis)
                _OpAnalysis.IsActivated = false;

            //active new op
            if (op == _MapOp.identify)
                _OpIdentify.IsActivated = true;
            else if (op == _MapOp.spatialquery)
                _OpSpatialQuery.IsActivated = true;
            else if (op == _MapOp.timequery)
                _OpTimeQuery.IsActivated = true;
            else if (op == _MapOp.analysis)
                _OpAnalysis.IsActivated = true;

            _CurrentOp = op;
        }

        private void map1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Z)
                map1.Zoom(0.5);
        }

        private void ChangeBaseMap(string newmap)
        {
            Storyboard sb = new Storyboard();
            DoubleAnimation daBingMap = new DoubleAnimation();
            daBingMap.Duration = TimeSpan.FromMilliseconds(500);
            DoubleAnimation daEMapZone = new DoubleAnimation();
            daEMapZone.Duration = TimeSpan.FromMilliseconds(500);

            ArcGISTiledMapServiceLayer layerEMapZone = map1.Layers[1] as ArcGISTiledMapServiceLayer;
            ESRI.ArcGIS.Client.Bing.TileLayer layerBingMap = map1.Layers[0] as ESRI.ArcGIS.Client.Bing.TileLayer;
            Storyboard.SetTarget(daBingMap, layerBingMap);
            Storyboard.SetTargetProperty(daBingMap, new PropertyPath("Opacity"));
            Storyboard.SetTarget(daEMapZone, layerEMapZone);
            Storyboard.SetTargetProperty(daEMapZone, new PropertyPath("Opacity"));

            sb.Children.Add(daEMapZone);
            sb.Children.Add(daBingMap);

            if (newmap == "Bing Maps")
            {
                daBingMap.From = 0;
                daBingMap.To = 1;
                daEMapZone.From = 1;
                daEMapZone.To = 0;
                map1.Layers[0].Visible = true;
                sb.Begin();
            }
            else if (newmap == "ArcGIS EMapZone")
            {
                daBingMap.From = 1;
                daBingMap.To = 0;
                daEMapZone.From = 0;
                daEMapZone.To = 1;
                sb.Begin();
                sb.Completed += (s, a) =>
                {
                    map1.Layers[0].Visible = false;
                };                
            }
        }

        private void ArcGISTiledMapServiceLayer_InitializationFailed(object sender, EventArgs e)
        {
            MessageBox.Show("EMapZone加载失败。");
        }

        private void ShowPopup()
        {
            Popup popup = new Popup();
            popup.Child = new Splashscreen();
            popup.IsOpen = true;

            BackgroundWorker backroungWorker = new BackgroundWorker();
            backroungWorker.DoWork += (s, a) =>
            {
                Random r = new Random();
                Thread.Sleep(4000+r.Next(2500));
            };
            backroungWorker.RunWorkerCompleted += (s, a) =>
            {
                this.Dispatcher.BeginInvoke(() =>
                {
                    popup.IsOpen = false; 
                    ApplicationBar.IsVisible = true;
                }
                );
            };
            backroungWorker.RunWorkerAsync();
        }

        //protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        //{
        //    base.OnNavigatedTo(e);
            
        //}

    }
}