﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Tasks;
using System.Collections.Generic;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Geometry;
using System.Linq;
using Microsoft.Phone.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using Microsoft.Phone.Shell;

namespace esridemo.Tools
{
    public class Analysis
    {
        private bool _isActivated;
        private FeatureLayer _FLayer;
        private GraphicsLayer _GLayer;
        private PhoneApplicationPage _PhoneAppPage;
        private Border _BusyIndicator;
        private Button _BtnBatchAnalysis;
        private Dictionary<int, double> _OrigianlOpacity = new Dictionary<int, double>(2);
        private bool _IsBatchAnalysis = false;
        private bool _FakeProgressing = false;//using by fakeanalysis, to stop transit to result page.
        private GeometryService _geometry = new GeometryService(App.Current.Resources["GeometryService"] as string);
        private Geoprocessor _gpDriveTime = new Geoprocessor(App.Current.Resources["GPDriveTime"] as string);
        private Geoprocessor _gpPopulation = new Geoprocessor(App.Current.Resources["GPPopulation"] as string);

        public Map map1 { get; set; }
        public bool IsActivated
        {
            get { return _isActivated; }
            set
            {
                if (_isActivated != value)
                {
                    _isActivated = value;
                    if (value)
                    {
                        (_PhoneAppPage.ApplicationBar.MenuItems[0] as ApplicationBarMenuItem).IsEnabled = false;
                        map1.Layers.Add(_GLayer);
                        _FLayer.MouseLeftButtonDown += _FLayer_MouseLeftButtonDown;
                        _BusyIndicator.Width = 250;
                        _BusyIndicator.Margin = new Thickness(0, 0, 0, 150);
                        //_BtnBatchAnalysis.Visibility = Visibility.Visible;
                        _OrigianlOpacity[0] = map1.Layers[0].Opacity;
                        _OrigianlOpacity[1] = map1.Layers[1].Opacity;

                        _gpDriveTime.ExecuteCompleted += gpDriveTime_ExecuteCompleted;
                        _gpDriveTime.Failed += (s, a) =>
                        {
                            MessageBox.Show("ServiceArea"+a.Error.Message + "\n" + a.UserState);
                            ChangeMapOpacity(-1);
                            _BusyIndicator.Visibility = Visibility.Collapsed;
                        };
                        _gpPopulation.ExecuteCompleted += gpPopulation_ExecuteCompleted;
                        _gpPopulation.Failed += (s, a) =>
                        {
                            ChangeMapOpacity(-1);
                            MessageBox.Show("Population"+a.Error.Message + "\n" + a.UserState);
                            _GLayer.Graphics.Clear();
                            _BusyIndicator.Visibility = Visibility.Collapsed;
                        };
                        _geometry.BufferCompleted += geometry_BufferCompleted;
                        _geometry.Failed += (s, a) =>
                        {
                            ChangeMapOpacity(-1);
                            MessageBox.Show("Buffer"+a.Error.Message + "\n" + a.UserState);
                            _BusyIndicator.Visibility = Visibility.Collapsed;
                        };
                    }
                    else
                    {
                        (_PhoneAppPage.ApplicationBar.MenuItems[0] as ApplicationBarMenuItem).IsEnabled = true;
                        _FLayer.MouseLeftButtonDown -= _FLayer_MouseLeftButtonDown;
                        map1.Layers.Remove(_GLayer);
                        (_BusyIndicator.Child as TextBlock).Text = "查询中......";
                        _BusyIndicator.Width = 140;
                        _BusyIndicator.Margin = new Thickness(0, 0, 0, 0);
                        _BusyIndicator.Visibility = Visibility.Collapsed;
                        //_BtnBatchAnalysis.Visibility = Visibility.Collapsed;
                        //set to normal
                        _FLayer.Graphics.Where(g => g.Attributes["Name"].ToString().Contains("回龙观")).ToList()[0].Symbol = App.Current.Resources["SuperMarket"] as MarkerSymbol;

                        ChangeMapOpacity(-1);

                        _gpPopulation.ExecuteCompleted -= gpPopulation_ExecuteCompleted;
                        _geometry.BufferCompleted -= geometry_BufferCompleted;
                        _gpDriveTime.ExecuteCompleted -= gpDriveTime_ExecuteCompleted;
                        CancelPreviousRequest();
                    }
                }
            }
        }

        void _FLayer_MouseLeftButtonDown(object sender, GraphicMouseButtonEventArgs e)
        {
            CancelPreviousRequest();
            _FakeProgressing = true;
            ChangeMapOpacity(0.3);

            map1.PanTo(e.Graphic.Geometry);


            App app = Application.Current as App;
            if (app.AppParameters.ContainsKey("AnalysisGraphic"))
                app.AppParameters["AnalysisGraphic"] = e.Graphic;
            else
                app.AppParameters.Add("AnalysisGraphic", e.Graphic);
            //prepare parameters for GP Service
            GraphicCollection gc = new GraphicCollection();
            gc.Add(e.Graphic);

            ////buffer the point as service area
            //Buffer(gc);

            CalculateServiceArea(gc);

            (_BusyIndicator.Child as TextBlock).Text = "查询服务区......";
            _BusyIndicator.Visibility = Visibility.Visible;            
        }

        private void Buffer(GraphicCollection gc)
        {
            BufferParameters bufferParams = new BufferParameters()
            {
                Unit = LinearUnit.Meter,
                BufferSpatialReference = map1.SpatialReference,
                OutSpatialReference = map1.SpatialReference
            };
            bufferParams.Features.AddRange(gc);
            bufferParams.Distances.Add(5000);
            bufferParams.UnionResults = true;
            

            _geometry.BufferAsync(bufferParams);
            (_BusyIndicator.Child as TextBlock).Text = "查询服务区......";
            _BusyIndicator.Visibility = Visibility.Visible;
        }

        /// <summary>
        /// find the service area of a given supermarket
        /// </summary>
        /// <param name="gc">the graphic collection contains the supermarket</param>
        private void CalculateServiceArea(GraphicCollection gc)
        {
            //drive time polygon as service area
            ESRI.ArcGIS.Client.Projection.WebMercator wm = new ESRI.ArcGIS.Client.Projection.WebMercator();
            GraphicCollection graphicCollection = new GraphicCollection();//using new gc to remain point(121000) stay on map
            foreach (Graphic g in gc)
            {
                Graphic graphic = new Graphic()
                {
                    Geometry=wm.ToGeographic(g.Geometry)
                };
                graphicCollection.Add(graphic);
            };

            FeatureSet fs = new FeatureSet(graphicCollection);
            List<GPParameter> parameters = new List<GPParameter>();
            parameters.Add(new GPFeatureRecordSetLayer("Input_Facilities", fs));
            parameters.Add(new GPString("Drive_Time_Values", "20 40"));
            //execute the GP Task
            _gpDriveTime.ExecuteAsync(parameters);
        }

        void geometry_BufferCompleted(object sender, GraphicsEventArgs e)
        {
            IList<Graphic> results = e.Results;

            foreach (Graphic graphic in results)
            {
                graphic.Symbol = new SimpleFillSymbol()
                {
                    Fill = new SolidColorBrush(Color.FromArgb(80, 0, 255, 0)),
                    BorderBrush=new SolidColorBrush(Colors.Green),
                    BorderThickness=1
                };
                _GLayer.Graphics.Add(graphic);
            }

            //query population
            //convert polygon from 102100 to 4326
            ESRI.ArcGIS.Client.Projection.WebMercator wm = new ESRI.ArcGIS.Client.Projection.WebMercator();
            Polygon polygon = wm.ToGeographic(_GLayer.Graphics[0].Geometry) as Polygon;

            List<GPParameter> parameters = new List<GPParameter>();
            parameters.Add(new GPFeatureRecordSetLayer("inputPoly", polygon));

            
            
            (_BusyIndicator.Child as TextBlock).Text = "查询服务区内人口数......";
            _gpPopulation.ExecuteAsync(parameters);
        }

            void gpDriveTime_ExecuteCompleted(object sender, GPExecuteCompleteEventArgs e)
            {
                ESRI.ArcGIS.Client.Projection.WebMercator wm = new ESRI.ArcGIS.Client.Projection.WebMercator();
                foreach (GPParameter parameter in e.Results.OutParameters)
                {
                    if (parameter is GPFeatureRecordSetLayer)
                    {
                        GPFeatureRecordSetLayer gpLayer = parameter as GPFeatureRecordSetLayer;

                        //40 min
                        gpLayer.FeatureSet.Features[0].Symbol = new SimpleFillSymbol()
                        {
                            Fill=new SolidColorBrush(Color.FromArgb(119,153,255,153)),
                            BorderBrush=new SolidColorBrush(Color.FromArgb(255,153,255,153)),
                            BorderThickness=2
                        };
                        gpLayer.FeatureSet.Features[0].Geometry.SpatialReference = new SpatialReference(4326);
                        gpLayer.FeatureSet.Features[0].Geometry = wm.FromGeographic(gpLayer.FeatureSet.Features[0].Geometry);
                        _GLayer.Graphics.Add(gpLayer.FeatureSet.Features[0]);
                        //20 min
                        gpLayer.FeatureSet.Features[1].Symbol = new SimpleFillSymbol()
                        {
                            Fill = new SolidColorBrush(Color.FromArgb(119, 153, 153, 255)),
                            BorderBrush = new SolidColorBrush(Color.FromArgb(255, 153, 153, 255)),
                            BorderThickness = 2
                        };
                        gpLayer.FeatureSet.Features[1].Geometry.SpatialReference = new SpatialReference(4326);
                        gpLayer.FeatureSet.Features[1].Geometry = wm.FromGeographic(gpLayer.FeatureSet.Features[1].Geometry);
                        _GLayer.Graphics.Add(gpLayer.FeatureSet.Features[1]);

                        map1.ZoomTo(gpLayer.FeatureSet.Features[0].Geometry);

                        //query population
                        //convert polygon from 102100 to 4326
                        Polygon polygon = wm.ToGeographic(_GLayer.Graphics[0].Geometry) as Polygon;

                        List<GPParameter> p = new List<GPParameter>();
                        p.Add(new GPFeatureRecordSetLayer("inputPoly", polygon));

                        (_BusyIndicator.Child as TextBlock).Text = "查询服务区内人口数......";
                        _gpPopulation.ExecuteAsync(p);
                    }
                }
            }

        void gpPopulation_ExecuteCompleted(object sender, GPExecuteCompleteEventArgs e)
        {
            _BusyIndicator.Visibility = Visibility.Collapsed;
            ChangeMapOpacity(-1);
            //screenshot of map control
            WriteableBitmap wb = new WriteableBitmap(map1, null);
            App app = Application.Current as App;
            if (app.AppParameters.ContainsKey("MapImage"))
                app.AppParameters["MapImage"] = wb;
            else
                app.AppParameters.Add("MapImage", wb);

            _GLayer.Graphics.Clear();            

            if (_IsBatchAnalysis)
            {
                //flash the alarm store
                _IsBatchAnalysis = false;
                _FLayer.Graphics.Where(g => g.Attributes["Name"].ToString().Contains("回龙观")).ToList()[0].Symbol = App.Current.Resources["SuperMarketAlarm"] as MarkerSymbol;
            }
            else
            {
                //navigate to report page
                Graphic result = (e.Results.OutParameters[0] as GPRecordSet).FeatureSet.Features[0];                
                _PhoneAppPage.NavigationService.Navigate(new Uri(string.Format("/Tools/AnalysisReport.xaml?population={0}", result.Attributes["SUM"].ToString()), UriKind.Relative));
            }                       
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        /// <param name="glayer">the featurelayer or graphicslayer hosts graphics to identify</param>
        public Analysis(Map map,PhoneApplicationPage page, Border busyindicator)
        {
            map1 = map;
            _PhoneAppPage = page;
            _BusyIndicator = busyindicator;
            _FLayer = map1.Layers["BusinessLayer"] as FeatureLayer;
            _GLayer = new GraphicsLayer();

            App app = Application.Current as App;
            if (app.AppParameters.ContainsKey("AllStores"))
                app.AppParameters["AllStores"] = _FLayer.Graphics;
            else
                app.AppParameters.Add("AllStores", _FLayer.Graphics);
        }

        void _BtnBatchAnalysis_Click(object sender, RoutedEventArgs e)
        {
            CancelPreviousRequest();

            _IsBatchAnalysis = true;
            ChangeMapOpacity(0.3);
            Buffer(_FLayer.Graphics);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="d">-1 to recover original opacity</param>
        private void ChangeMapOpacity(double d)
        {
            if (d == -1)
            {
                map1.Layers[0].Opacity = _OrigianlOpacity[0];
                map1.Layers[1].Opacity = _OrigianlOpacity[1];
            }
            else
            {
                map1.Layers[0].Opacity = _OrigianlOpacity[0] * d;
                map1.Layers[1].Opacity = _OrigianlOpacity[1] * d;
            }
        }

        private void CancelPreviousRequest()
        {
            _FakeProgressing = false;
            _geometry.CancelAsync();
            _gpPopulation.CancelAsync();
            _gpDriveTime.CancelAsync();
            _GLayer.ClearGraphics();
        }
        
    }
}
