﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using System.Linq;
using Microsoft.Phone.Controls;
using System.Windows.Threading;

namespace esridemo.Tools
{
    public class Identify
    {
        private bool _isActivated;
        private bool _isNormalMapTap;//indicate tap gesture is fired normally by user or innormally by GraphicsMouseLeftButtonDown 
        private ElementLayer _elementLayer;//InfoWindow layer        
        public Map _map1 { get; set; }
        public GraphicsLayer GLayer { get; set; }//supermarket layer
        public bool IsActivated
        {
            get { return _isActivated; }
            set
            {
                if (_isActivated != value)
                {
                    _isActivated = value;
                    if (value)
                    {
                        if(_elementLayer==null)
                            _elementLayer = new ElementLayer();
                        if (!_map1.Layers.Contains(_elementLayer))
                            _map1.Layers.Add(_elementLayer);
                        GLayer.MouseLeftButtonDown += GLayer_MouseLeftButtonDown;
                        _map1.MapGesture += map1_MapGesture;
                    }
                    else
                    {
                        if (_map1.Layers.Contains(_elementLayer))
                            _map1.Layers.Remove(_elementLayer);
                        _map1.MapGesture -= map1_MapGesture;
                        GLayer.MouseLeftButtonDown -= GLayer_MouseLeftButtonDown;
                        if (GLayer.SelectedGraphics.Count() > 0)
                            GLayer.SelectedGraphics.ToList()[0].UnSelect();
                        _elementLayer.Children.Clear();
                    }
                }
            }
        }

        void map1_MapGesture(object sender, Map.MapGestureEventArgs e)
        {
            if (_isNormalMapTap)
            {
                if (e.Gesture == GestureType.Tap)
                {
                    if (GLayer.SelectedGraphics.Count() > 0)//if has infowindow displayed
                    {
                        GLayer.SelectedGraphics.ToList()[0].UnSelect();
                        _elementLayer.Children.Clear();
                    }
                }
            }
        }

        void GLayer_MouseLeftButtonDown(object sender, GraphicMouseButtonEventArgs e)
        {
            _isNormalMapTap = false;
            DispatcherTimer dt = new DispatcherTimer()//determin for isNormalMapTap
            {
                Interval=TimeSpan.FromMilliseconds(500),
            };
            dt.Tick += (s, a) =>
            {
                _isNormalMapTap = true;
                dt.Stop();
            };
            dt.Start();
            

            if (GLayer.SelectedGraphics.Count() > 0)
                GLayer.SelectedGraphics.ToList()[0].UnSelect();

            Graphic g = e.Graphic;
            //info window
            //remove other infowindow
            _elementLayer.Children.Clear();
            InfoWindow infoWindow = new InfoWindow(_elementLayer, e.Graphic);
            ESRI.ArcGIS.Client.ElementLayer.SetEnvelope(infoWindow, new Envelope((e.Graphic.Geometry as MapPoint), (e.Graphic.Geometry as MapPoint)));
            _elementLayer.Children.Add(infoWindow);

            e.Graphic.Select();

            if (Math.Abs(_map1.Resolution - 19.1092570712683) < 0.0001)
                _map1.PanTo(e.Graphic.Geometry);
            else
            {
                double ratio = 19.1092570712683 / _map1.Resolution;
                // Get the current extent of the map.
                ESRI.ArcGIS.Client.Geometry.Envelope myEnvelope = _map1.Extent;

                // Get the current center of the maps extent.
                ESRI.ArcGIS.Client.Geometry.MapPoint centerMapPoint = myEnvelope.GetCenter();

                // Calculate the X,Y coordinates for the new map extent center.
                double X = ((e.Graphic.Geometry as MapPoint).X - ratio * centerMapPoint.X) / (1 - ratio);
                double Y = ((e.Graphic.Geometry as MapPoint).Y - ratio * centerMapPoint.Y) / (1 - ratio);
                ESRI.ArcGIS.Client.Geometry.MapPoint newMapPoint = new ESRI.ArcGIS.Client.Geometry.MapPoint(X, Y);

                // Zoom in and re-center the map.
                _map1.ZoomToResolution(19.1092570712683, newMapPoint);

            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        /// <param name="glayer">the featurelayer or graphicslayer hosts graphics to identify</param>
        public Identify(Map map,GraphicsLayer glayer)
        {
            _map1 = map;
            GLayer = glayer;
        }
    }
}
