﻿using System.Windows.Media;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using System;
using System.Windows.Threading;

namespace esridemo.Tools
{
    public class SpatialQuery
    {
        private bool _isActivated;
        private bool _usingFreeHand = false;
        private Draw _draw;
        private bool _IsQuerying;//to determin if a query is in progress, using to cancel a query.
        private Border _busyIndicator;
        private GraphicsLayer _GLayer;//used to display the polygon and result points
        public Map map1 { get; set; }
        public CheckBox _chkboxDrawPattern;
        public bool IsActivated
        {
            get { return _isActivated; }
            set
            {
                if (_isActivated != value)
                {
                    _isActivated = value;
                    if (value)
                    {
                        if(_GLayer==null)
                          _GLayer=new GraphicsLayer();
                        if (!map1.Layers.Contains(_GLayer))
                            map1.Layers.Add(_GLayer);

                        //_chkboxDrawPattern.Visibility = Visibility.Visible;
                        if(_draw==null)
                            _draw = new Draw(map1);
                        _usingFreeHand = true;
                        _draw.DrawMode = _usingFreeHand ? DrawMode.Freehand : DrawMode.Polygon;
                        _draw.LineSymbol = new SimpleLineSymbol(Colors.Red, 4);
                        _draw.FillSymbol = new SimpleFillSymbol()
                        {
                            BorderBrush=new SolidColorBrush(Colors.Red),
                            BorderThickness=3,
                            Fill=new SolidColorBrush(Color.FromArgb(50,255,0,0))
                        };
                        _draw.DrawComplete += _draw_DrawComplete;
                        _draw.IsEnabled = true;
                    }
                    else
                    {                        
                        _GLayer.ClearGraphics();
                        if (map1.Layers.Contains(_GLayer))
                            map1.Layers.Remove(_GLayer);
                        _GLayer = null;
                        _draw.IsEnabled = false;
                        _draw.DrawComplete -= _draw_DrawComplete;
                        _draw=null;
                        _busyIndicator.Visibility = Visibility.Collapsed;
                        _IsQuerying = false;
                        _chkboxDrawPattern.Visibility = Visibility.Collapsed;
                    }
                }
            }
        }

        void _draw_DrawComplete(object sender, DrawEventArgs e)
        {
            _IsQuerying = true;
            Polygon polygon = null;
            if (_usingFreeHand)//geometry is freehand polyline
            {
                Polyline polyline = e.Geometry as Polyline;
                ESRI.ArcGIS.Client.Geometry.PointCollection pc = polyline.Paths[0];
                pc.Add(pc[0]);
                polygon = new Polygon()
                {
                    SpatialReference = map1.SpatialReference,
                };
                polygon.Rings.Add(pc);
            }
            else//geometry is polygon
            {
                polygon = e.Geometry as Polygon;
            }

            _GLayer.Graphics.Clear();
            Graphic g = new Graphic()
            {
                Geometry = polygon,
                Symbol = new SimpleFillSymbol()
                {
                    Fill=new SolidColorBrush(Color.FromArgb(33,255,0,0)),
                    BorderBrush=new SolidColorBrush(Colors.Red),
                    BorderThickness=2
                }
            };
            _GLayer.Graphics.Add(g);//display the geometry created by Draw object

            QueryTask queryTask = new QueryTask(App.Current.Resources["BusinessLayer"] as string);
            Query query = new Query();
            //102100 to 4326
            ESRI.ArcGIS.Client.Projection.WebMercator wm = new ESRI.ArcGIS.Client.Projection.WebMercator();
            query.Geometry = wm.ToGeographic(polygon);
            query.OutFields.AddRange(new string[] { "*" });//return all attributes fields
            query.SpatialRelationship = SpatialRelationship.esriSpatialRelIntersects;
            queryTask.ExecuteCompleted += new System.EventHandler<QueryEventArgs>(queryTask_ExecuteCompleted);
            queryTask.Failed += (s, a) =>
            {
                MessageBox.Show("查询失败:SDS暂不提供空间查询功能" + a.Error.Message);
            };
            _busyIndicator.Visibility = Visibility.Visible;
            DispatcherTimer dt = new DispatcherTimer()
            {
                Interval=TimeSpan.FromMilliseconds(1000)
            };
            dt.Tick += (s, a) =>
            {
                dt.Stop();
                queryTask.ExecuteAsync(query);
            };
            dt.Start();  
        }

        void queryTask_ExecuteCompleted(object sender, QueryEventArgs e)
        {
            _busyIndicator.Visibility = Visibility.Collapsed;
            if (e.FeatureSet.Features.Count == 0)//typeof(e.FeatureSet.Features)==GraphicCollection
            {
                MessageBox.Show("未选中任何店面");
            }
            else
            {
                int iDayStart = 24;
                int iDayEnd = 30;
                double dTotal, dSum;
                dTotal = dSum = 0;
                //sum of selected stores in last 7 days 
                foreach (Graphic g in e.FeatureSet.Features)
                {
                    for (int i = iDayStart; i <= iDayEnd; i++)//11.23--11.30
                    {
                        string strDay = string.Empty;
                        if (i < 10)
                            strDay = "0" + i.ToString();
                        else
                            strDay = i.ToString();

                        dSum += Double.Parse(g.Attributes["D11" + strDay].ToString());
                    }
                }

                //sum of all stores in last seven days
                foreach (Graphic g in (map1.Layers["BusinessLayer"] as FeatureLayer).Graphics)
                {
                    for (int i = iDayStart; i <= iDayEnd; i++)//11.23--11.30
                    {
                        string strDay = string.Empty;
                        if (i < 10)
                            strDay = "0" + i.ToString();
                        else
                            strDay = i.ToString();

                        dTotal += Double.Parse(g.Attributes["D11" + strDay].ToString());
                    }
                }

                string str = string.Format("选中了{0}个店面\n\n过去一周营业额之和：{1}\n\n占所有店面过去一周总营业额({2})的{3}%\n\n是否查看图表详情？",
                e.FeatureSet.Features.Count.ToString(),
                dSum.ToString("###,###.0"),
                dTotal.ToString("###,###.0"),
                (dSum / dTotal * 100).ToString("###,###.0"));

                if (_IsQuerying)
                {
                    if (MessageBox.Show(str,
                    "查询结果", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                    {
                        //add selected graphics to app level, so spatialquerychart.xaml could retrieve them
                        App app = Application.Current as App;
                        if (app.AppParameters.ContainsKey("QueryedGraphics"))
                            app.AppParameters["QueryedGraphics"] = e.FeatureSet.Features;
                        else
                            app.AppParameters.Add("QueryedGraphics", e.FeatureSet.Features);

                        (app.AppParameters["MainPage"] as PhoneApplicationPage).NavigationService.Navigate(new Uri(string.Format("/Tools/SpatialQueryChart.xaml?total={0}&sum={1}", dTotal.ToString(), dSum.ToString()), UriKind.Relative));
                    }
                    _GLayer.ClearGraphics();
                }
            }
        }


        public SpatialQuery(Map map,Border busyindicator)
        {
            map1 = map;
            _busyIndicator = busyindicator;

            _chkboxDrawPattern = new CheckBox()
            {
                Content = "自由手势？",
                FontSize = 20.0,
                FontWeight=FontWeights.Bold,
                Foreground = new SolidColorBrush(Color.FromArgb(200,255,0,0)),
                HorizontalAlignment = HorizontalAlignment.Left,
                VerticalAlignment = VerticalAlignment.Top,
                Visibility = Visibility.Collapsed,
            };
            _chkboxDrawPattern.Click += new RoutedEventHandler(_chkboxDrawPattern_Click);
            (map1.Parent as Grid).Children.Add(_chkboxDrawPattern);  
        }

        void _chkboxDrawPattern_Click(object sender, RoutedEventArgs e)
        {
            CheckBox chkbox = sender as CheckBox;
            _usingFreeHand = chkbox.IsChecked == true ? true : false;
            _draw.DrawMode = _usingFreeHand ? DrawMode.Freehand : DrawMode.Polygon;
        }
    }
}
