﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;
using System.Linq;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace esridemo.Tools
{
    public class TimeQuery
    {
        private bool _isActivated;
        DispatcherTimer _dTimer;//use to control autoplay
        private Border _Border;//stackpanel used to hold the slider and textblock
        private CheckBox _chkboxAutoPlay;//auto paly animation?
        private Slider _slider;//time slider
        private TextBlock _textblock;//date textblock
        private FeatureLayer _FLayer;//business layer
        private GraphicsLayer _GLayerSymbol;//do nothing with Business FeatureLayer, using another graphicslayer to display symobls
        private GraphicsLayer _GLayerText;//becuase the binding didn't work well in wp7 api, so using another textsymbol in a graphicslayer
        //instead of a custom markersymbol with a text in it(using binding). 
        private double _salesmin, _salesmax;//use to determine symbol size
        private enum SymbolSize
        {
            small=20,
            middle=40,
            large=60
        }
        public Map map1 { get; set; }
        public bool IsActivated
        {
            get { return _isActivated; }
            set
            {
                if (_isActivated != value)
                {
                    _isActivated = value;
                    if (value)
                    {                        
                        SlidePanel(Visibility.Visible, TimeSpan.FromMilliseconds(300));
                        _FLayer.Visible = false;
                        map1.Layers.Add(_GLayerSymbol);
                        map1.Layers.Add(_GLayerText);
                        InitSymbols();
                        _chkboxAutoPlay.Visibility = Visibility.Visible;
                        _slider.ValueChanged += _slider_ValueChanged;
                        map1.ExtentChanged += map1_ExtentChanged;
                    }
                    else
                    {
                        SlidePanel(Visibility.Collapsed, TimeSpan.FromMilliseconds(300));
                        _FLayer.Visible = true;
                        map1.Layers.Remove(_GLayerText);
                        map1.Layers.Remove(_GLayerSymbol);
                        UnInitSymbols();
                        _chkboxAutoPlay.Visibility = Visibility.Collapsed;
                        map1.ExtentChanged -= map1_ExtentChanged;
                        
                        _slider.ValueChanged -= _slider_ValueChanged;
                        _chkboxAutoPlay.IsChecked = false;
                        _dTimer.Stop();
                    }
                }
            }
        }        

        public TimeQuery(Map map)
        {
            map1 = map;
            _GLayerText = new GraphicsLayer();
            _GLayerSymbol= new GraphicsLayer();
            _FLayer = map1.Layers["BusinessLayer"] as FeatureLayer;

            _slider = new Slider()
            {
                Height=84,
                Width=360,
                Minimum=1,
                Maximum=30,
                SmallChange=1,
                LargeChange=5,
                Margin=new Thickness(0,0,0,-30)
            };

            _textblock = new TextBlock()
            {
                Text = "时间:11月1日",
                Foreground=new SolidColorBrush(Colors.White),
                FontSize=20,
                FontWeight=FontWeights.Bold,
                Margin=new Thickness(20,0,0,0),
            };
            StackPanel sp = new StackPanel()
            {
                HorizontalAlignment=HorizontalAlignment.Center,
                VerticalAlignment=VerticalAlignment.Center,
            };
            sp.Children.Add(_slider);

            //auto play box
            _chkboxAutoPlay = new CheckBox()
            {
                Content = "连续查看",
                FontSize = 20,
                FontWeight=FontWeights.Bold,
                Foreground = new SolidColorBrush(Colors.White),
                Visibility = Visibility.Collapsed,
                Margin = new Thickness(90, -22, 0, 0),
            };

            _dTimer = new DispatcherTimer()
            {
                Interval = TimeSpan.FromMilliseconds(500)
            };
            _dTimer.Tick += (sender, args) =>
            {
                if (_slider.Value == 30)
                    _slider.Value = 1;
                else
                    _slider.Value += 1;
            };

            _chkboxAutoPlay.Click += (s, a) =>
            {
                CheckBox chkbox = s as CheckBox;
                if (chkbox.IsChecked == true)
                {
                    _dTimer.Start();
                }
                else
                {
                    _dTimer.Stop();
                }
            };
            StackPanel sp1 = new StackPanel()
            {
                Orientation = Orientation.Horizontal
            };
            sp1.Children.Add(_textblock);
            sp1.Children.Add(_chkboxAutoPlay);
            sp.Children.Add(sp1);

            _Border = new Border()
            {
                Background=new SolidColorBrush(Color.FromArgb(225,0,0,0)),
                Width=400,
                Height=0,
                CornerRadius= new CornerRadius(10),
                BorderBrush=new SolidColorBrush(Colors.White),
                BorderThickness=new Thickness(5),
                //Visibility=Visibility.Collapsed,
                VerticalAlignment=VerticalAlignment.Top,
            };
            _Border.Child = sp;

            (map1.Parent as Grid).Children.Add(_Border);


            _salesmin = _salesmax = 0;
            //find day minsales and maxsales in all graphics and all days
            for (int i = 1; i <= 30; i++)
            {
                string strDay = string.Empty;
                if (i < 10)
                    strDay = "0" + i.ToString();
                else
                    strDay = i.ToString();
                double fieldmax = (from graphic in _FLayer.Graphics
                                   select graphic).Max(a => (double)a.Attributes["D11" + strDay]);
                _salesmax = _salesmax > fieldmax ? _salesmax : fieldmax;
                double fieldmin = (from graphic in _FLayer.Graphics
                                   select graphic).Min(a => (double)a.Attributes["D11" + strDay]);
                _salesmin = _salesmin < fieldmin ? _salesmin : fieldmin;
            }
        }

        void _slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            int day = (int)e.NewValue;
            //change text
            _textblock.Text = string.Format("时间:11月{0}日", day.ToString());

            ChangeSymbolAndText(day);
        }

        private void InitSymbols()
        {
            foreach (Graphic g in _FLayer.Graphics)
            {
                //determin symbol size
                double size = -1;
                if ((double)g.Attributes["D1101"] <= (_salesmax + _salesmin)/2 * 1 / 3)
                    size = (double)SymbolSize.small;
                else if ((double)g.Attributes["D1101"] > (_salesmax + _salesmin) / 2 * 1 / 3 && (double)g.Attributes["D1101"] <= (_salesmax + _salesmin) / 2 * 2 / 3)
                    size = (double)SymbolSize.middle;
                else
                    size = (double)SymbolSize.large;

                //change Businesslayer symbol to custom symbols and display in _GLayerSymbol for each store
                Graphic gsymbol = new Graphic()
                {
                    Geometry = g.Geometry,
                    Symbol = new PictureMarkerSymbol()
                    {
                        Source = new BitmapImage(new Uri("../Images/Dollar.png", UriKind.Relative)),
                    }
                };
                
                (gsymbol.Symbol as PictureMarkerSymbol).Height = (gsymbol.Symbol as PictureMarkerSymbol).Width = size;
                (gsymbol.Symbol as PictureMarkerSymbol).OffsetX = (gsymbol.Symbol as PictureMarkerSymbol).OffsetY = size / 2;
                g.Attributes.Add("symbol", gsymbol);

                //add a graphic with textsymbol, which display sales, to _GLayerText for each store
                Graphic gtext = new Graphic()
                {
                    Geometry = g.Geometry,
                    Symbol = new TextSymbol()
                    {
                        Foreground = new SolidColorBrush(Color.FromArgb(225,255,255,255)),
                        Text = double.Parse(g.Attributes["D1101"].ToString()).ToString("###,###"),
                        //OffsetX=-size/2+10,
                        //OffsetY=-size/2+10,
                        //FontSize = size == (double)SymbolSize.large ? size-10:size
                        OffsetX = -size / 2+15,
                        OffsetY = -size / 2+15,
                        FontSize=30
                    }
                };
                gtext.Symbol.ControlTemplate = (App.Current.Resources["LegendTextSymbol"] as TextSymbol).ControlTemplate;
                g.Attributes.Add("text", gtext);

                _GLayerText.Graphics.Add(gtext);
                _GLayerSymbol.Graphics.Add(gsymbol); 
            }
            //_GLayerSymbol.MouseLeftButtonDown += new GraphicsLayer.MouseButtonEventHandler(_GLayerSymbol_MouseLeftButtonDown);
            _GLayerText.Visible = false;
        }

        /// <summary>
        /// when clicked symbol, popup store name
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _GLayerSymbol_MouseLeftButtonDown(object sender, GraphicMouseButtonEventArgs e)
        {
            
        }

        private void UnInitSymbols()
        {
            foreach (Graphic g in _FLayer.Graphics)
            {
                g.Attributes.Remove("text");
                g.Attributes.Remove("symbol");
            }
            _GLayerText.ClearGraphics();
            _GLayerSymbol.ClearGraphics();
        }

        /// <summary>
        /// change symbols and text according to current day
        /// </summary>
        /// <param name="day"></param>
        private void ChangeSymbolAndText(int day)
        {
            string strDay = string.Empty;
            if (day < 10)
                strDay = "D11"+"0" + day.ToString();
            else
                strDay = "D11" + day.ToString();
            
            foreach (Graphic g in _FLayer.Graphics)
            {
                //determin symbol size
                double size = -1;
                if ((double)g.Attributes[strDay] <= (_salesmax - _salesmin) * 1 / 3)
                    size = (double)SymbolSize.small;
                else if ((double)g.Attributes[strDay] > (_salesmax - _salesmin) * 1 / 3 && (double)g.Attributes[strDay] <= (_salesmax - _salesmin) * 2 / 3)
                    size = (double)SymbolSize.middle;
                else
                    size = (double)SymbolSize.large;

                PictureMarkerSymbol symbol = (g.Attributes["symbol"] as Graphic).Symbol as PictureMarkerSymbol;
                symbol.Height = symbol.Width = size;
                symbol.OffsetX = symbol.OffsetY = size / 2;

                TextSymbol text = (g.Attributes["text"] as Graphic).Symbol as TextSymbol;
                text.Text = double.Parse(g.Attributes[strDay].ToString()).ToString("###,###");
                text.OffsetX = -size / 2 + 10;
                text.OffsetY = -size / 2 + 10;
                //text.FontSize = size == (double)SymbolSize.small ? size : size - 10;
            }
        }

        void map1_ExtentChanged(object sender, ExtentEventArgs e)
        {
            if (map1.Resolution > 50.2185141425366)//38
                _GLayerText.Visible = false;
            else
                _GLayerText.Visible = true;
        }

        /// <summary>
        /// display or hide the time panel with a slide animation
        /// </summary>
        /// <param name="visible"></param>
        /// <param name="duration"></param>
        private void SlidePanel(Visibility visible, TimeSpan duration)
        {
            DoubleAnimation da = new DoubleAnimation();
            da.Duration = duration;
            if (visible == Visibility.Visible)
            {
                //_Border.Visibility = Visibility.Visible;
                da.From = 0;
                da.To = 100;
            }
            else
            {
                //_Border.Visibility = Visibility.Collapsed;
                da.From = 100;
                da.To = 0;
            }
            Storyboard sb = new Storyboard();
            Storyboard.SetTarget(da, _Border);
            Storyboard.SetTargetProperty(da, new PropertyPath("Height"));
            sb.Children.Add(da);
            sb.Begin();
        }
    }
}
